package de.bokeh.icfp2008;

/**
 * Strategy for exploring the map.
 * <p>
 * Currently a misnomer because it just heads straigt for home,
 * ignoring any obstacles.
 */
public class ExploreStrategy implements Strategy {

    private static final int MAX_ANGLE_OFF = 6;
    
    private final World world;
    
    public ExploreStrategy(World world) {
        this.world = world;
    }

    @Override
    public void getCommands(AccelerationState[] accel, TurningState[] turn) {
        
        // stay near top speed all the time
        if (world.getCurrentSpeedRatio() < 0.9)
            accel[0] = AccelerationState.ACCELERATING;
        else
            accel[0] = AccelerationState.ROLLING;
        
        // move directly toward home
        
        MovingObjectState m = world.getCurrentTelemetry().move;
        
        TurningState t = TurningState.STRAIGHT;
        int rdir = world.getCurrentTelemetry().turningState.compareTo(TurningState.STRAIGHT);

        double targetDir = m.getAngleTo(0, 0);
        if (targetDir < -MAX_ANGLE_OFF) {
            t = TurningState.LEFT;
        } else if (targetDir > MAX_ANGLE_OFF) {
            t = TurningState.RIGHT;
        } else {
            // correct direction, steer straight
            if (rdir > 0)
                t = TurningState.LEFT;
            else if (rdir < 0)
                t = TurningState.RIGHT;
        }

        // but avoid craters and martians!
        // FIXME: does not seem to work
        if (world.isCraterOrMartianAhead() && t == TurningState.STRAIGHT && world.getCurrentTelemetry().turningState == TurningState.STRAIGHT) {
            t = TurningState.RIGHT;
        }
        
        turn[0] = t;
    }
    
}
