package de.bokeh.icfp2008;

/**
 * A fixed object: a crater, a boulder, or the home base.
 */
public class FixedObject {

    public enum Kind { BOULDER, CRATER, HOME };

    public final Kind kind;

    public final double x;
    public final double y;
    public final double r;

    public FixedObject(Kind kind, double x, double y, double r) {
        this.kind = kind;
        this.x = x;
        this.y = y;
        this.r = r;
    }
    
    public String toString() {
        return kind + "{x=" + x + ", y=" + y + ", r=" + r + "}";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((kind == null) ? 0 : kind.hashCode());
        long temp;
        temp = Double.doubleToLongBits(r);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(x);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(y);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final FixedObject other = (FixedObject) obj;
        if (kind == null) {
            if (other.kind != null)
                return false;
        } else if (!kind.equals(other.kind))
            return false;
        if (Double.doubleToLongBits(r) != Double.doubleToLongBits(other.r))
            return false;
        if (Double.doubleToLongBits(x) != Double.doubleToLongBits(other.x))
            return false;
        if (Double.doubleToLongBits(y) != Double.doubleToLongBits(other.y))
            return false;
        return true;
    }

}
