package de.bokeh.icfp2008;

/**
 * Utility functions.
 */
public class Geometry {

    /**
     * Get angle to specific point.
     * 
     * @param sx x-coordinate of source object
     * @param sy y-coordinate of source object
     * @param dir direction of source object
     * @param dx x-coordinate of destination point
     * @param dy y-coordinate of destination point
     * @return The closest angle from source to dest. If the source object is at the destination point,
     * 0 is returned. The return value is always between 180 and -180. Negative values are for clockwise turning.
     * If the destination is exactly in the opposite direction, 180 is returned.
     */
    public static double getAngleTo(double sx, double sy, double dir, double dx, double dy) {
        if (sx == dx && sy == dy)
            return 0.0;
        
        double delta_x = dx - sx;
        double delta_y = dy - sy;
        double angleRad = Math.atan2(delta_y, delta_x);
        double angleDeg = angleRad / Math.PI * 180;
        return normalizeDir(dir - angleDeg);
    }
    
    
    /**
     * Normalize direction to be in the range (-180 .. 180].
     * @param dir the direction
     * @return The normalized direction.
     */
    public static double normalizeDir(double dir) {
        while (dir > 180)
            dir -= 360;
        while (dir <= -180)
            dir += 360;
        return dir;
    }
    
    /**
     * @return Returns the distance between two points.
     */
    public static double distance(double x1, double y1, double x2, double y2) {
        double dx = x1 - x2;
        double dy = y1 - y2;
        return Math.sqrt(dx * dx + dy * dy);
    }
    
    public static boolean lineCircleIntersect(double x1, double y1, double dir, double x0, double y0, double r) {
        double x2 = x1 + 100 * Math.sin(dir * Math.PI / 180.0);
        double y2 = y1 + 100 * Math.cos(dir * Math.PI / 180.0);
        double n = Math.abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1));
        double d = Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
        double dist = n/d;
        if(dist > r)return false;
        double d1 = Math.sqrt((x0-x1)*(x0-x1)+(y0-y1)*(y0-y1));
        if((d1-r) > d)return false;
        double d2 = Math.sqrt((x0-x2)*(x0-x2)+(y0-y2)*(y0-y2));
        if((d2-r) > d)return false;
        return true;
    }
}
