package de.bokeh.icfp2008;

/**
 * Message the rover sends to the controller.
 */
public class Message {

	public enum Kind { CRASH, CRATER, KILLED, SUCCESS, END_OF_RUN, TELEMETRY };
	
	public final Kind kind;
	
	/**
	 * Milliseconds since start of run.
	 */
	public final int timeStamp;
	
	private final Telemetry telemetry;

	private final int score;
	
	private Message(Kind k, int ts, int score, Telemetry tele) {
		kind = k;
		timeStamp = ts;
		this.score = score;
		telemetry = tele;
	}
	
	public static Message crash(int timeStamp) {
		return new Message(Kind.CRASH, timeStamp, 0, null);
	}
	
	public static Message crater(int timeStamp) {
		return new Message(Kind.CRATER, timeStamp, 0, null);
	}
	
	public static Message killed(int timeStamp) {
		return new Message(Kind.KILLED, timeStamp, 0, null);
	}
	
	public static Message success(int timeStamp) {
		return new Message(Kind.SUCCESS, timeStamp, 0, null);
	}
	
	public static Message endOfRun(int timeStamp, int score) {
		return new Message(Kind.END_OF_RUN, timeStamp, score, null);
	}
	
	public static Message telemetry(int timeStamp, Telemetry t) {
		return new Message(Kind.TELEMETRY, timeStamp, 0, t);
	}
	
	public int getScore() {
		if (kind == Kind.END_OF_RUN)
			return score;
		throw new UnsupportedOperationException();
	}
	
	public Telemetry getTelemetry() {
		if (kind == Kind.TELEMETRY)
			return telemetry;
		throw new UnsupportedOperationException();
	}
	
	public String toString() {
		switch (kind) {
		case TELEMETRY: return "Tele " + timeStamp + " " + telemetry;
		case END_OF_RUN: return "End " + timeStamp + " " + score;
		default: return kind + " " + timeStamp + " " + score;
		}
	}
}
