package de.bokeh.icfp2008;

/**
 * State of a moving object (Rover or martian).
 */
public class MovingObjectState {

    public final double x;

    public final double y;

    /**
     * Degrees counterclockwise from the x-axis.
     */
    public final double dir;

    /**
     * Speed in meters per second.
     */
    public final double speed;
    
    public MovingObjectState(double x, double y, double dir, double speed) {
        this.x = x;
        this.y = y;
        // normalize direction
        while (dir > 180)
            dir -= 360;
        while (dir <= -180)
            dir += 360;
        this.dir = dir;
        this.speed = speed;
    }

    public String toString() {
        return "x=" + x + ", y=" + y + ", dir=" + dir + ", speed=" + speed;
    }
    
    /**
     * @return Return the angle to a point on the map.
     */
    public double getAngleTo(double x, double y) {
        return Geometry.getAngleTo(this.x, this.y, dir, x, y);
    }
}
