package de.bokeh.icfp2008;


import java.util.*;


public class RandomStrategy implements Strategy {

    private final World world;
    private final Random rng;
    
    
    public RandomStrategy(World world) {
        this.world = world;
        rng = new Random();
    }
    
    
    @Override
    public void getCommands(AccelerationState[] accel, TurningState[] turn) {
        double spd = world.getCurrentSpeedRatio();
        AccelerationState a = AccelerationState.ROLLING;
        if (spd > 0.75)
            a = AccelerationState.BRAKING;
        else if (spd < 0.4)
            a = AccelerationState.ACCELERATING;
        
        TurningState t0 = world.getCurrentTelemetry().turningState;
        TurningState t1 = TurningState.STRAIGHT;
        int rnd = rng.nextInt(8);
        switch (t0) {
        case STRAIGHT:
            switch (rnd) {
            case 6: t1 = TurningState.LEFT; break;
            case 7: t1 = TurningState.RIGHT; break;
            }
            break;
        case LEFT:
            if (rnd < 4)
                t1 = TurningState.RIGHT;
            break;
        case RIGHT:
            if (rnd < 4)
                t1 = TurningState.LEFT;
            break;
        case HARD_LEFT:
            t1 = TurningState.RIGHT;
            break;
        case HARD_RIGHT:
            t1 = TurningState.LEFT;
            break;
        }
        accel[0] = a;
        turn[0] = t1;
    }

}
