package de.bokeh.icfp2008;

import java.util.*;

/**
 * Contents of a telemetry message.
 */
public class Telemetry {

    public final AccelerationState accelerationState;

    public final TurningState turningState;

    public final MovingObjectState move;

    private final List<FixedObject> objects;

    private final List<MovingObjectState> martians;

    public Telemetry(AccelerationState a, TurningState t, MovingObjectState m) {
        accelerationState = a;
        turningState = t;
        move = m;
        objects = new ArrayList<FixedObject>();
        martians = new ArrayList<MovingObjectState>();
    }

    public List<FixedObject> getObjects() {
        return Collections.unmodifiableList(objects);
    }
    
    public List<MovingObjectState> getMartians() {
        return Collections.unmodifiableList(martians);
    }
    
    public void addObject(FixedObject o) {
        objects.add(o);
    }

    public void addMartian(MovingObjectState m) {
        martians.add(m);
    }


    public String toString() {
        StringBuilder b = new StringBuilder();
        b.append("Telemetry{");
        b.append(accelerationState);
        b.append(", ");
        b.append(turningState);
        b.append(", ");
        b.append(move);
        for (FixedObject o : objects) {
            b.append(", [");
            b.append(o);
            b.append("]");
        }
        for (MovingObjectState o : martians) {
            b.append(", [Martian{");
            b.append(o);
            b.append("}]");
        }
        return b.toString();
    }
}
